from typing import MutableMapping, Union


class DotList(list):
    def __getitem__(self, index):
        value = super().__getitem__(index)
        if isinstance(value, dict):
            return Dot(value)
        return value


class Dot(dict):

    def __getattr__(self, item) -> Union['Dot', DotList]:
        value = self[item]
        if isinstance(value, dict):
            return Dot(value)
        elif isinstance(value, list):
            return DotList(value)
        return value


from setuptools import setup

with open("README.md", "r",encoding="utf-8") as fh:
    long_description = fh.read()



setup(
    name='durable_dot_dict',
    version='0.0.10',
    description='Durable Dot Dictionary',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Risto Kowaczewski',
    packages=['durable_dot_dict'],
    install_requires=[
        'dotdict_parser'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=['durable_dot_dict'],
    include_package_data=True,
    python_requires=">=3.10",
)



from durable_dot_dict.dot import Dot
from durable_dot_dict.dotdict import DotDict


def test_mapper_left1():
    data = DotDict() << {
        "b.a": None,
        'c.a': 1,
        'd.a': "",
        'e.a': {"b": 1},
        'f.a': [{"a": 1}, 1],
        "list": [1],
        "set": set()
    }

    assert Dot(data).b.a is None

    data.map({
        "ala_ma_kota": "test"
    }) << [
        ("b.a", 'ala_ma_kota'),
        ("d.a", 'none_existent')
    ]

    assert Dot(data).b.a == "test"
    assert Dot(data).d.a == ""


def test_mapper_left2():
    data = DotDict() << {
        "b.a": None,
        'c.a': 1,
        'd.a': "",
        'e.a': {"b": 1},
        'f.a': [{"a": 1}, 1],
        "list": [1],
        "set": set()
    }

    assert Dot(data).b.a is None

    data.map({
        "ala_ma_kota": "test"
    }) << {
        "b.a": 'ala_ma_kota',
        "d.a": 'none_existent'
    }

    assert Dot(data).b.a == "test"
    assert Dot(data).d.a == ""


def test_mapper_right1():
    data = DotDict() << {
        "b.a": None,
        'c.a': 1,
        'd.a': "",
        'e.a': {"b": 1},
        'f.a': [{"a": 1}, 1],
        "list": [1],
        "set": set()
    }

    assert Dot(data).b.a is None

    new_value = data.map({"ala_ma_kota": "test"}) >> [('none', 'a'), ('b', 'b'), ('c.a', 'c')]

    assert new_value == {'ala_ma_kota': 'test', 'b': {'a': None}, 'c': 1}


def test_mapper_right2():
    data = DotDict() << {
        "b.a": None,
        'c.a': 1,
        'd.a': "",
        'e.a': {"b": 1},
        'f.a': [{"a": 1}, 1],
        "list": [1],
        "set": set()
    }

    assert Dot(data).b.a is None

    # Use set to map

    new_value = data.map({"ala_ma_kota": "test"}) >> {"b.a", "c.a"}

    assert new_value == {'ala_ma_kota': 'test', 'b': {'a': None}, 'c': {"a": 1}}
